/*

  System call tracer

  Copyright (C) 1991 K.Abe

  Patch for Human3.02 by Tachibana Eriko

*/

#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <sys/xunistd.h>

#include "dosmac.h"
#include "trace.h"

#define	FLINE_TRAP_VECTOR	11
#define HUMAN_VERSION		0x302

typedef void (*handler)(void);

static	 void	trace( const char* , const char** );
static	 void	convert_slash_to_backslash( char* );

extern void New_handler( void );


handler	Old_handler;
FILE	*Stream;
int	Count = 0;


int main( int argc , char *argv[] )
{
    char	*command_name;
    char	**command_argv;
    char	*ofile = NULL;
    int		i;

    if( argc <= 1 )
	usage();

    for( i = 1 ; i < argc && argv[i][0] == '-' ; i++ )
    {
	switch( argv[i][1] )
	{
	    case 'o':
		if( argv[i][2] )
		    ofile = argv[i] + 2;
		else
		    ofile = argv[ ++i ];
		break;
	    default:
		usage();
		break;
	}
    }
    if ( argc <= i )
	usage();

    if( HUMAN_VERSION != (unsigned short)_dos_vernum() )
	version_error();

    if( ofile != NULL )
    {
	if( ( Stream = fopen( ofile , "wt" ) ) == NULL )
	{
	    perror( "fopen" );
	    exit(1);
	}
    }
    else
	Stream = stdout;

    /* COOKED [h̃LN^foCXȊO̓oCi[hɂ */
    if( _fddb[ fileno( Stream )].iomd & ( _DEV_CDEV | _DEV_RAW ) != _DEV_CDEV )
	fmode( Stream , _IOBINARY );

    Initialize_argument_information();
    command_name =  argv[i];
    command_argv = &argv[i];
    convert_slash_to_backslash( command_name );
    trace( command_name , (void *)command_argv );
    return 0;
}


static void trace( const char *command_name , const char *command_argv[] )
{
    int	rc;

    setup_doscall_handler();
    rc = spawnvp( P_WAIT , command_name , (void *)command_argv );
    restore_doscall_handler();

    if( rc == -1 )
    {
	perror( "spawnvp" );
	exit(1);
    }
    fprintf( Stream , "Exit code %d (%d System Calls)\n" , rc , Count );
}

static void convert_slash_to_backslash( char *command_name )
{
    do
    {
	if( *command_name == '/')
	    *command_name = '\\';
    }
    while( *command_name++ );
}

/* EOF */
